@@ -1,5 +1,4 @@
-/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
- * applicable.
+/* Copyright 2000-2004 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,34 +28,37 @@ static apr_status_t proc_mutex_no_tryacquire(apr_proc_mutex_t *new_mutex)
     return APR_ENOTIMPL;
 }
 
+#if APR_HAS_POSIXSEM_SERIALIZE || APR_HAS_FCNTL_SERIALIZE || \
+    APR_HAS_PROC_PTHREAD_SERIALIZE || APR_HAS_SYSVSEM_SERIALIZE
+static apr_status_t proc_mutex_no_child_init(apr_proc_mutex_t **mutex,
+                                             apr_pool_t *cont,
+                                             const char *fname)
+{
+    return APR_SUCCESS;
+}
+#endif    
+
 #if APR_HAS_POSIXSEM_SERIALIZE
 
 #ifndef SEM_FAILED
 #define SEM_FAILED (-1)
 #endif
 
-static void proc_mutex_posix_setup(void)
-{
-}
-
 static apr_status_t proc_mutex_posix_cleanup(void *mutex_)
 {
-    apr_proc_mutex_t *mutex=mutex_;
-    apr_status_t stat = APR_SUCCESS;
+    apr_proc_mutex_t *mutex = mutex_;
     
-    if (mutex->interproc->filedes != -1) {
-        if (sem_close((sem_t *)mutex->interproc->filedes) < 0) {
-            stat = errno;
-        }
+    if (sem_close(mutex->psem_interproc) < 0) {
+        return errno;
     }
-    return stat;
+
+    return APR_SUCCESS;
 }    
 
 static apr_status_t proc_mutex_posix_create(apr_proc_mutex_t *new_mutex,
                                             const char *fname)
 {
     sem_t *psem;
-    apr_status_t stat;
     char semname[31];
     apr_time_t now;
     unsigned long sec;
@@ -64,7 +66,6 @@ static apr_status_t proc_mutex_posix_create(apr_proc_mutex_t *new_mutex,
     
     new_mutex->interproc = apr_palloc(new_mutex->pool,
                                       sizeof(*new_mutex->interproc));
-    new_mutex->interproc->filedes = -1;
     /*
      * This bogusness is to follow what appears to be the
      * lowest common denominator in Posix semaphore naming:
@@ -102,13 +103,11 @@ static apr_status_t proc_mutex_posix_create(apr_proc_mutex_t *new_mutex,
     }
 
     if (psem == (sem_t *)SEM_FAILED) {
-        stat = errno;
-        proc_mutex_posix_cleanup(new_mutex);
-        return stat;
+        return errno;
     }
     /* Ahhh. The joys of Posix sems. Predelete it... */
     sem_unlink((const char *) semname);
-    new_mutex->interproc->filedes = (int)psem;	/* Ugg */
+    new_mutex->psem_interproc = psem;
     new_mutex->fname = apr_pstrdup(new_mutex->pool, semname);
     apr_pool_cleanup_register(new_mutex->pool, (void *)new_mutex,
                               apr_proc_mutex_cleanup, 
@@ -118,9 +117,7 @@ static apr_status_t proc_mutex_posix_create(apr_proc_mutex_t *new_mutex,
 
 static apr_status_t proc_mutex_posix_acquire(apr_proc_mutex_t *mutex)
 {
-    int rc;
-
-    if ((rc = sem_wait((sem_t *)mutex->interproc->filedes)) < 0) {
+    if (sem_wait(mutex->psem_interproc) < 0) {
         return errno;
     }
     mutex->curr_locked = 1;
@@ -129,23 +126,16 @@ static apr_status_t proc_mutex_posix_acquire(apr_proc_mutex_t *mutex)
 
 static apr_status_t proc_mutex_posix_release(apr_proc_mutex_t *mutex)
 {
-    int rc;
-
     mutex->curr_locked = 0;
-    if ((rc = sem_post((sem_t *)mutex->interproc->filedes)) < 0) {
+    if (sem_post(mutex->psem_interproc) < 0) {
+        /* any failure is probably fatal, so no big deal to leave
+         * ->curr_locked at 0. */
         return errno;
     }
     return APR_SUCCESS;
 }
 
-static apr_status_t proc_mutex_posix_child_init(apr_proc_mutex_t **mutex,
-                                                apr_pool_t *cont,
-                                                const char *fname)
-{
-    return APR_SUCCESS;
-}
-
-const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_posix_methods =
+static const apr_proc_mutex_unix_lock_methods_t mutex_posixsem_methods =
 {
 #if APR_PROCESS_LOCK_IS_GLOBAL || !APR_HAS_THREADS || defined(POSIXSEM_IS_GLOBAL)
     APR_PROCESS_LOCK_MECH_IS_GLOBAL,
@@ -157,7 +147,7 @@ const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_posix_methods =
     proc_mutex_no_tryacquire,
     proc_mutex_posix_release,
     proc_mutex_posix_cleanup,
-    proc_mutex_posix_child_init,
+    proc_mutex_no_child_init,
     "posixsem"
 };
 
@@ -245,12 +235,7 @@ static apr_status_t proc_mutex_sysv_release(apr_proc_mutex_t *mutex)
     return APR_SUCCESS;
 }
 
-static apr_status_t proc_mutex_sysv_child_init(apr_proc_mutex_t **mutex, apr_pool_t *cont, const char *fname)
-{
-    return APR_SUCCESS;
-}
-
-const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_sysv_methods =
+static const apr_proc_mutex_unix_lock_methods_t mutex_sysv_methods =
 {
 #if APR_PROCESS_LOCK_IS_GLOBAL || !APR_HAS_THREADS || defined(SYSVSEM_IS_GLOBAL)
     APR_PROCESS_LOCK_MECH_IS_GLOBAL,
@@ -262,7 +247,7 @@ const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_sysv_methods =
     proc_mutex_no_tryacquire,
     proc_mutex_sysv_release,
     proc_mutex_sysv_cleanup,
-    proc_mutex_sysv_child_init,
+    proc_mutex_no_child_init,
     "sysvsem"
 };
 
@@ -270,10 +255,6 @@ const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_sysv_methods =
 
 #if APR_HAS_PROC_PTHREAD_SERIALIZE
 
-static void proc_mutex_proc_pthread_setup(void)
-{
-}
-
 static apr_status_t proc_mutex_proc_pthread_cleanup(void *mutex_)
 {
     apr_proc_mutex_t *mutex=mutex_;
@@ -285,13 +266,22 @@ static apr_status_t proc_mutex_proc_pthread_cleanup(void *mutex_)
             rv = errno;
 #endif
             return rv;
-        } 
-        if (munmap((caddr_t)mutex->pthread_interproc, sizeof(pthread_mutex_t))){
-            return errno;
         }
     }
+    /* curr_locked is set to -1 until the mutex has been created */
+    if (mutex->curr_locked != -1) {
+        if ((rv = pthread_mutex_destroy(mutex->pthread_interproc))) {
+#ifdef PTHREAD_SETS_ERRNO
+            rv = errno;
+#endif
+            return rv;
+        }
+    }
+    if (munmap((caddr_t)mutex->pthread_interproc, sizeof(pthread_mutex_t))) {
+        return errno;
+    }
     return APR_SUCCESS;
-}    
+}
 
 static apr_status_t proc_mutex_proc_pthread_create(apr_proc_mutex_t *new_mutex,
                                                    const char *fname)
@@ -311,9 +301,13 @@ static apr_status_t proc_mutex_proc_pthread_create(apr_proc_mutex_t *new_mutex,
                                        PROT_READ | PROT_WRITE, MAP_SHARED,
                                        fd, 0); 
     if (new_mutex->pthread_interproc == (pthread_mutex_t *) (caddr_t) -1) {
+        close(fd);
         return errno;
     }
     close(fd);
+
+    new_mutex->curr_locked = -1; /* until the mutex has been created */
+
     if ((rv = pthread_mutexattr_init(&mattr))) {
 #ifdef PTHREAD_SETS_ERRNO
         rv = errno;
@@ -326,16 +320,18 @@ static apr_status_t proc_mutex_proc_pthread_create(apr_proc_mutex_t *new_mutex,
         rv = errno;
 #endif
         proc_mutex_proc_pthread_cleanup(new_mutex);
+        pthread_mutexattr_destroy(&mattr);
         return rv;
     }
 
-#ifdef HAVE_PTHREAD_MUTEX_ROBUST
+#ifdef HAVE_PTHREAD_MUTEXATTR_SETROBUST_NP
     if ((rv = pthread_mutexattr_setrobust_np(&mattr, 
                                                PTHREAD_MUTEX_ROBUST_NP))) {
 #ifdef PTHREAD_SETS_ERRNO
         rv = errno;
 #endif
         proc_mutex_proc_pthread_cleanup(new_mutex);
+        pthread_mutexattr_destroy(&mattr);
         return rv;
     }
     if ((rv = pthread_mutexattr_setprotocol(&mattr, PTHREAD_PRIO_INHERIT))) {
@@ -343,18 +339,22 @@ static apr_status_t proc_mutex_proc_pthread_create(apr_proc_mutex_t *new_mutex,
         rv = errno;
 #endif
         proc_mutex_proc_pthread_cleanup(new_mutex);
+        pthread_mutexattr_destroy(&mattr);
         return rv;
     }
-#endif /* HAVE_PTHREAD_MUTEX_ROBUST */
+#endif
 
     if ((rv = pthread_mutex_init(new_mutex->pthread_interproc, &mattr))) {
 #ifdef PTHREAD_SETS_ERRNO
         rv = errno;
 #endif
         proc_mutex_proc_pthread_cleanup(new_mutex);
+        pthread_mutexattr_destroy(&mattr);
         return rv;
     }
 
+    new_mutex->curr_locked = 0; /* mutex created now */
+
     if ((rv = pthread_mutexattr_destroy(&mattr))) {
 #ifdef PTHREAD_SETS_ERRNO
         rv = errno;
@@ -363,7 +363,6 @@ static apr_status_t proc_mutex_proc_pthread_create(apr_proc_mutex_t *new_mutex,
         return rv;
     }
 
-    new_mutex->curr_locked = 0;
     apr_pool_cleanup_register(new_mutex->pool,
                               (void *)new_mutex,
                               apr_proc_mutex_cleanup, 
@@ -379,7 +378,7 @@ static apr_status_t proc_mutex_proc_pthread_acquire(apr_proc_mutex_t *mutex)
 #ifdef PTHREAD_SETS_ERRNO
         rv = errno;
 #endif
-#ifdef HAVE_PTHREAD_MUTEX_ROBUST
+#ifdef HAVE_PTHREAD_MUTEXATTR_SETROBUST_NP
         /* Okay, our owner died.  Let's try to make it consistent again. */
         if (rv == EOWNERDEAD) {
             pthread_mutex_consistent_np(mutex->pthread_interproc);
@@ -410,14 +409,7 @@ static apr_status_t proc_mutex_proc_pthread_release(apr_proc_mutex_t *mutex)
     return APR_SUCCESS;
 }
 
-static apr_status_t proc_mutex_proc_pthread_child_init(apr_proc_mutex_t **mutex,
-                                            apr_pool_t *cont, 
-                                            const char *fname)
-{
-    return APR_SUCCESS;
-}
-
-const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_proc_pthread_methods =
+static const apr_proc_mutex_unix_lock_methods_t mutex_proc_pthread_methods =
 {
     APR_PROCESS_LOCK_MECH_IS_GLOBAL,
     proc_mutex_proc_pthread_create,
@@ -425,7 +417,7 @@ const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_proc_pthread_method
     proc_mutex_no_tryacquire,
     proc_mutex_proc_pthread_release,
     proc_mutex_proc_pthread_cleanup,
-    proc_mutex_proc_pthread_child_init,
+    proc_mutex_no_child_init,
     "pthread"
 };
 
@@ -462,10 +454,8 @@ static apr_status_t proc_mutex_fcntl_cleanup(void *mutex_)
         if (status != APR_SUCCESS)
             return status;
     }
-    if (mutex->interproc) { /* if it was opened successfully */
-        apr_file_close(mutex->interproc);
-    }
-    return APR_SUCCESS;
+        
+    return apr_file_close(mutex->interproc);
 }    
 
 static apr_status_t proc_mutex_fcntl_create(apr_proc_mutex_t *new_mutex,
@@ -488,7 +478,6 @@ static apr_status_t proc_mutex_fcntl_create(apr_proc_mutex_t *new_mutex,
     }
  
     if (rv != APR_SUCCESS) {
-        proc_mutex_fcntl_cleanup(new_mutex);
         return rv;
     }
 
@@ -529,14 +518,7 @@ static apr_status_t proc_mutex_fcntl_release(apr_proc_mutex_t *mutex)
     return APR_SUCCESS;
 }
 
-static apr_status_t proc_mutex_fcntl_child_init(apr_proc_mutex_t **mutex,
-                                                apr_pool_t *pool, 
-                                                const char *fname)
-{
-    return APR_SUCCESS;
-}
-
-const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_fcntl_methods =
+static const apr_proc_mutex_unix_lock_methods_t mutex_fcntl_methods =
 {
 #if APR_PROCESS_LOCK_IS_GLOBAL || !APR_HAS_THREADS || defined(FCNTL_IS_GLOBAL)
     APR_PROCESS_LOCK_MECH_IS_GLOBAL,
@@ -548,7 +530,7 @@ const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_fcntl_methods =
     proc_mutex_no_tryacquire,
     proc_mutex_fcntl_release,
     proc_mutex_fcntl_cleanup,
-    proc_mutex_fcntl_child_init,
+    proc_mutex_no_child_init,
     "fcntl"
 };
 
@@ -558,10 +540,6 @@ const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_fcntl_methods =
 
 static apr_status_t proc_mutex_flock_release(apr_proc_mutex_t *);
 
-static void proc_mutex_flock_setup(void)
-{
-}
-
 static apr_status_t proc_mutex_flock_cleanup(void *mutex_)
 {
     apr_status_t status;
@@ -655,14 +633,13 @@ static apr_status_t proc_mutex_flock_child_init(apr_proc_mutex_t **mutex,
     rv = apr_file_open(&new_mutex->interproc, new_mutex->fname,
                        APR_WRITE, 0, new_mutex->pool);
     if (rv != APR_SUCCESS) {
-        proc_mutex_flock_cleanup(new_mutex);
         return rv;
     }
     *mutex = new_mutex;
     return APR_SUCCESS;
 }
 
-const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_flock_methods =
+static const apr_proc_mutex_unix_lock_methods_t mutex_flock_methods =
 {
 #if APR_PROCESS_LOCK_IS_GLOBAL || !APR_HAS_THREADS || defined(FLOCK_IS_GLOBAL)
     APR_PROCESS_LOCK_MECH_IS_GLOBAL,
@@ -682,21 +659,13 @@ const apr_proc_mutex_unix_lock_methods_t apr_proc_mutex_unix_flock_methods =
 
 void apr_proc_mutex_unix_setup_lock(void)
 {
-#if APR_HAS_POSIXSEM_SERIALIZE
-    proc_mutex_posix_setup();
-#endif
+    /* setup only needed for sysvsem and fnctl */
 #if APR_HAS_SYSVSEM_SERIALIZE
     proc_mutex_sysv_setup();
 #endif
-#if APR_HAS_PROC_PTHREAD_SERIALIZE
-    proc_mutex_proc_pthread_setup();
-#endif
 #if APR_HAS_FCNTL_SERIALIZE
     proc_mutex_fcntl_setup();
 #endif
-#if APR_HAS_FLOCK_SERIALIZE
-    proc_mutex_flock_setup();
-#endif
 }
 
 static apr_status_t proc_mutex_choose_method(apr_proc_mutex_t *new_mutex, apr_lockmech_e mech)
@@ -704,50 +673,50 @@ static apr_status_t proc_mutex_choose_method(apr_proc_mutex_t *new_mutex, apr_lo
     switch (mech) {
     case APR_LOCK_FCNTL:
 #if APR_HAS_FCNTL_SERIALIZE
-        new_mutex->inter_meth = &apr_proc_mutex_unix_fcntl_methods;
+        new_mutex->inter_meth = &mutex_fcntl_methods;
 #else
         return APR_ENOTIMPL;
 #endif
         break;
     case APR_LOCK_FLOCK:
 #if APR_HAS_FLOCK_SERIALIZE
-        new_mutex->inter_meth = &apr_proc_mutex_unix_flock_methods;
+        new_mutex->inter_meth = &mutex_flock_methods;
 #else
         return APR_ENOTIMPL;
 #endif
         break;
     case APR_LOCK_SYSVSEM:
 #if APR_HAS_SYSVSEM_SERIALIZE
-        new_mutex->inter_meth = &apr_proc_mutex_unix_sysv_methods;
+        new_mutex->inter_meth = &mutex_sysv_methods;
 #else
         return APR_ENOTIMPL;
 #endif
         break;
     case APR_LOCK_POSIXSEM:
 #if APR_HAS_POSIXSEM_SERIALIZE
-        new_mutex->inter_meth = &apr_proc_mutex_unix_posix_methods;
+        new_mutex->inter_meth = &mutex_posixsem_methods;
 #else
         return APR_ENOTIMPL;
 #endif
         break;
     case APR_LOCK_PROC_PTHREAD:
 #if APR_HAS_PROC_PTHREAD_SERIALIZE
-        new_mutex->inter_meth = &apr_proc_mutex_unix_proc_pthread_methods;
+        new_mutex->inter_meth = &mutex_proc_pthread_methods;
 #else
         return APR_ENOTIMPL;
 #endif
         break;
     case APR_LOCK_DEFAULT:
 #if APR_USE_FLOCK_SERIALIZE
-        new_mutex->inter_meth = &apr_proc_mutex_unix_flock_methods;
+        new_mutex->inter_meth = &mutex_flock_methods;
 #elif APR_USE_SYSVSEM_SERIALIZE
-        new_mutex->inter_meth = &apr_proc_mutex_unix_sysv_methods;
+        new_mutex->inter_meth = &mutex_sysv_methods;
 #elif APR_USE_FCNTL_SERIALIZE
-        new_mutex->inter_meth = &apr_proc_mutex_unix_fcntl_methods;
+        new_mutex->inter_meth = &mutex_fcntl_methods;
 #elif APR_USE_PROC_PTHREAD_SERIALIZE
-        new_mutex->inter_meth = &apr_proc_mutex_unix_proc_pthread_methods;
+        new_mutex->inter_meth = &mutex_proc_pthread_methods;
 #elif APR_USE_POSIXSEM_SERIALIZE
-        new_mutex->inter_meth = &apr_proc_mutex_unix_posix_methods;
+        new_mutex->inter_meth = &mutex_posixsem_methods;
 #else
         return APR_ENOTIMPL;
 #endif
@@ -796,13 +765,8 @@ APR_DECLARE(apr_status_t) apr_proc_mutex_create(apr_proc_mutex_t **mutex,
     apr_proc_mutex_t *new_mutex;
     apr_status_t rv;
 
-    new_mutex = (apr_proc_mutex_t *)apr_pcalloc(pool,
-                                                sizeof(apr_proc_mutex_t));
-
-    new_mutex->pool  = pool;
-#if APR_HAS_SYSVSEM_SERIALIZE || APR_HAS_FCNTL_SERIALIZE || APR_HAS_FLOCK_SERIALIZE || APR_HAS_POSIXSEM_SERIALIZE
-    new_mutex->interproc = NULL;
-#endif
+    new_mutex = apr_pcalloc(pool, sizeof(apr_proc_mutex_t));
+    new_mutex->pool = pool;
 
     if ((rv = proc_mutex_create(new_mutex, mech, fname)) != APR_SUCCESS)
         return rv;
@@ -845,13 +809,18 @@ APR_DECLARE(const char *) apr_proc_mutex_name(apr_proc_mutex_t *mutex)
 
 APR_DECLARE(const char *) apr_proc_mutex_lockfile(apr_proc_mutex_t *mutex)
 {
-    /* posix sems use the fname field but don't use a file,
-     * so be careful 
-     */
-    if (!strcmp(mutex->meth->name, "flock") ||
-        !strcmp(mutex->meth->name, "fcntl")) {
+    /* POSIX sems use the fname field but don't use a file,
+     * so be careful. */
+#if APR_HAS_FLOCK_SERIALIZE
+    if (mutex->meth == &mutex_flock_methods) {
         return mutex->fname;
     }
+#endif
+#if APR_HAS_FCNTL_SERIALIZE
+    if (mutex->meth == &mutex_fcntl_methods) {
+        return mutex->fname;
+    }
+#endif
     return NULL;
 }
 
