@@ -1,9 +1,8 @@
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+/* Copyright 2000-2004 The Apache Software Foundation
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -98,49 +97,6 @@ static apr_status_t get_remote_addr(apr_socket_t *sock)
     }
 }
 
-APR_DECLARE(apr_status_t) apr_sockaddr_port_set(apr_sockaddr_t *sockaddr,
-                                       apr_port_t port)
-{
-    sockaddr->port = port;
-    /* XXX IPv6: assumes sin_port and sin6_port at same offset */
-    sockaddr->sa.sin.sin_port = htons(port);
-    return APR_SUCCESS;
-}
-
-/* XXX assumes IPv4... I don't think this function is needed anyway
- * since we have apr_sockaddr_info_get(), but we need to clean up Apache's 
- * listen.c a bit more first.
- */
-APR_DECLARE(apr_status_t) apr_sockaddr_ip_set(apr_sockaddr_t *sockaddr,
-                                         const char *addr)
-{
-    apr_uint32_t ipaddr;
-    
-    if (!strcmp(addr, APR_ANYADDR)) {
-        sockaddr->sa.sin.sin_addr.s_addr = htonl(INADDR_ANY);
-        return APR_SUCCESS;
-    }
-    
-    ipaddr = inet_addr(addr);
-    if (ipaddr == (apr_uint32_t)-1) {
-#ifdef WIN32
-        return WSAEADDRNOTAVAIL;
-#else
-        return errno;
-#endif
-    }
-    
-    sockaddr->sa.sin.sin_addr.s_addr = ipaddr;
-    return APR_SUCCESS;
-}
-
-APR_DECLARE(apr_status_t) apr_sockaddr_port_get(apr_port_t *port,
-                                       apr_sockaddr_t *sockaddr)
-{
-    *port = sockaddr->port;
-    return APR_SUCCESS;
-}
-
 APR_DECLARE(apr_status_t) apr_sockaddr_ip_get(char **addr,
                                          apr_sockaddr_t *sockaddr)
 {
@@ -336,6 +292,14 @@ static apr_status_t call_resolver(apr_sockaddr_t **sa,
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = family;
     hints.ai_socktype = SOCK_STREAM;
+#ifdef HAVE_GAI_ADDRCONFIG
+    if (family == AF_UNSPEC) {
+        /* By default, only look up addresses using address types for
+         * which a local interface is configured, i.e. no IPv6 if no
+         * IPv6 interfaces configured. */
+        hints.ai_flags = AI_ADDRCONFIG;
+    }
+#endif
     if(hostname == NULL) {
 #ifdef AI_PASSIVE 
         /* If hostname is NULL, assume we are trying to bind to all
@@ -345,6 +309,17 @@ static apr_status_t call_resolver(apr_sockaddr_t **sa,
         /* getaddrinfo according to RFC 2553 must have either hostname
          * or servname non-NULL.
          */
+#ifdef OSF1
+        /* The Tru64 5.0 getaddrinfo() can only resolve services given
+         * by the name listed in /etc/services; a numeric or unknown
+         * servname gets an EAI_SERVICE error.  So just resolve the
+         * appropriate anyaddr and fill in the port later. */
+        hostname = family == AF_INET6 ? "::" : "0.0.0.0";
+        servname = NULL;
+#ifdef AI_NUMERICHOST
+        hints.ai_flags |= AI_NUMERICHOST;
+#endif
+#else
 #ifdef _AIX
         /* But current AIX getaddrinfo() doesn't like servname = "0";
          * the "1" won't hurt since we use the port parameter to fill
@@ -354,10 +329,18 @@ static apr_status_t call_resolver(apr_sockaddr_t **sa,
             servname = "1";
         }
         else
-#endif
+#endif /* _AIX */
         servname = apr_itoa(p, port);
+#endif /* OSF1 */
     }
     error = getaddrinfo(hostname, servname, &hints, &ai_list);
+#ifdef HAVE_GAI_ADDRCONFIG
+    if (error == EAI_BADFLAGS && family == AF_UNSPEC) {
+        /* Retry with no flags if AI_ADDRCONFIG was rejected. */
+        hints.ai_flags = 0;
+        error = getaddrinfo(hostname, servname, &hints, &ai_list);
+    }
+#endif
     if (error) {
 #ifndef WIN32
         if (error == EAI_SYSTEM) {
@@ -612,7 +595,6 @@ APR_DECLARE(apr_status_t) apr_getnameinfo(char **hostname,
         IN6_IS_ADDR_V4MAPPED(&sockaddr->sa.sin6.sin6_addr)) {
         struct sockaddr_in tmpsa;
         tmpsa.sin_family = AF_INET;
-        tmpsa.sin_port = 0;
         tmpsa.sin_addr.s_addr = ((apr_uint32_t *)sockaddr->ipaddr_ptr)[3];
 #ifdef SIN6_LEN
         tmpsa.sin_len = sizeof(tmpsa);
@@ -722,7 +704,7 @@ APR_DECLARE(apr_status_t) apr_getservbyname(apr_sockaddr_t *sockaddr,
         sockaddr->sa.sin.sin_port = se->s_port;
         return APR_SUCCESS;
     }
-    return APR_ENOENT;
+    return errno;
 }
 
 #define V4MAPPED_EQUAL(a,b)                                   \
