@@ -50,7 +50,9 @@ static apr_int16_t get_kqueue_revent(apr_int16_t event, apr_int16_t flags)
         rv |= APR_POLLIN;
     if (event & EVFILT_WRITE)
         rv |= APR_POLLOUT;
-    if (flags & EV_ERROR || flags & EV_EOF)
+    if (flags & EV_EOF)
+        rv |= APR_POLLHUP;
+    if (flags & EV_ERROR)
         rv |= APR_POLLERR;
 
     return rv;
