@@ -1,5 +1,4 @@
-/* Copyright 2000-2005 The Apache Software Foundation or its licensors, as
- * applicable.
+/* Copyright 2000-2004 The Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,6 +17,7 @@
 #include "apr_strings.h"
 #include "apr_portable.h"
 #include "apr_signal.h"
+#include "apr_random.h"
 
 APR_DECLARE(apr_status_t) apr_procattr_create(apr_procattr_t **new,
                                               apr_pool_t *pool)
@@ -109,18 +109,16 @@ APR_DECLARE(apr_status_t) apr_procattr_child_in_set(apr_procattr_t *attr,
                                                     apr_file_t *child_in,
                                                     apr_file_t *parent_in)
 {
-    apr_status_t rv = APR_SUCCESS;
-
     if (attr->child_in == NULL && attr->parent_in == NULL)
-        rv = apr_file_pipe_create(&attr->child_in, &attr->parent_in, attr->pool);
-    
-    if (child_in != NULL && rv == APR_SUCCESS)
-        rv = apr_file_dup2(attr->child_in, child_in, attr->pool);
+        apr_file_pipe_create(&attr->child_in, &attr->parent_in, attr->pool);
 
-    if (parent_in != NULL && rv == APR_SUCCESS)
-        rv = apr_file_dup2(attr->parent_in, parent_in, attr->pool);
+    if (child_in != NULL)
+        apr_file_dup2(attr->child_in, child_in, attr->pool);
 
-    return rv;
+    if (parent_in != NULL)
+        apr_file_dup2(attr->parent_in, parent_in, attr->pool);
+
+    return APR_SUCCESS;
 }
 
 
@@ -128,18 +126,16 @@ APR_DECLARE(apr_status_t) apr_procattr_child_out_set(apr_procattr_t *attr,
                                                      apr_file_t *child_out,
                                                      apr_file_t *parent_out)
 {
-    apr_status_t rv = APR_SUCCESS;
-
     if (attr->child_out == NULL && attr->parent_out == NULL)
-        rv = apr_file_pipe_create(&attr->child_out, &attr->parent_out, attr->pool);
+        apr_file_pipe_create(&attr->child_out, &attr->parent_out, attr->pool);
 
-    if (child_out != NULL && rv == APR_SUCCESS)
-        rv = apr_file_dup2(attr->child_out, child_out, attr->pool);
+    if (child_out != NULL)
+        apr_file_dup2(attr->child_out, child_out, attr->pool);
 
-    if (parent_out != NULL && rv == APR_SUCCESS)
-        rv = apr_file_dup2(attr->parent_out, parent_out, attr->pool);
+    if (parent_out != NULL)
+        apr_file_dup2(attr->parent_out, parent_out, attr->pool);
 
-    return rv;
+    return APR_SUCCESS;
 }
 
 
@@ -147,18 +143,16 @@ APR_DECLARE(apr_status_t) apr_procattr_child_err_set(apr_procattr_t *attr,
                                                      apr_file_t *child_err,
                                                      apr_file_t *parent_err)
 {
-    apr_status_t rv = APR_SUCCESS;
-
     if (attr->child_err == NULL && attr->parent_err == NULL)
-        rv = apr_file_pipe_create(&attr->child_err, &attr->parent_err, attr->pool);
+        apr_file_pipe_create(&attr->child_err, &attr->parent_err, attr->pool);
 
-    if (child_err != NULL && rv == APR_SUCCESS)
-        rv = apr_file_dup2(attr->child_err, child_err, attr->pool);
+    if (child_err != NULL)
+        apr_file_dup2(attr->child_err, child_err, attr->pool);
 
-    if (parent_err != NULL && rv == APR_SUCCESS)
-        rv = apr_file_dup2(attr->parent_err, parent_err, attr->pool);
+    if (parent_err != NULL)
+        apr_file_dup2(attr->parent_err, parent_err, attr->pool);
 
-    return rv;
+    return APR_SUCCESS;
 }
 
 
@@ -200,6 +194,8 @@ APR_DECLARE(apr_status_t) apr_proc_fork(apr_proc_t *proc, apr_pool_t *pool)
         proc->out = NULL;
         proc->err = NULL;
 
+        apr_random_after_fork(proc);
+
         return APR_INCHILD;
     }
 
@@ -292,14 +288,6 @@ APR_DECLARE(apr_status_t) apr_proc_create(apr_proc_t *new,
                                           apr_pool_t *pool)
 {
     int i;
-    const char * const empty_envp[] = {NULL};
-
-    if (!env) { /* Specs require an empty array instead of NULL;
-                 * Purify will trigger a failure, even if many
-                 * implementations don't.
-                 */
-        env = empty_envp;
-    }
 
     new->in = attr->parent_in;
     new->err = attr->parent_err;
